#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# Author:        sebastian.piec@desy.de
# Last modified: 2017, October 2
# ----------------------------------------------------------------------

"""Errors related functionality
"""

import traceback
import logging

from PyQt5 import QtWidgets

from petra_camera.main_window import APP_NAME

logger = logging.getLogger(APP_NAME)


# ----------------------------------------------------------------------
def report_error(err, parent=None, simplify=False):
    """Send error message to the logging object and show user-friendly
    dialog box.
    """
    logger.exception(err)

    msg = str(err) if simplify else "{}\n\n{}".format(str(err).capitalize(),
                                                          str(traceback.format_exc()))

    QtWidgets.QMessageBox.warning(parent, "Exception", msg, QtWidgets.QMessageBox.Ok)




# Created by matveyev at 22.02.2022

from distutils.util import strtobool

from PyQt5 import QtWidgets, QtCore

from petra_camera.utils.functions import refresh_combo_box
from petra_camera.gui.CameraSettings_ui import Ui_CameraSettings


class CameraSettings(QtWidgets.QWidget):

    delete_me = QtCore.pyqtSignal(int)
    new_name = QtCore.pyqtSignal(int, str)

    def __init__(self, parent, my_id, settings_node=None):

        super(CameraSettings, self).__init__()
        self._ui = Ui_CameraSettings()
        self._ui.setupUi(self)

        self.my_id = my_id

        self._ui.cmb_camera_type.currentTextChanged.connect(self._switch_camera_type)
        self._ui.cmb_motor_type.currentTextChanged.connect(self._switch_motor_type)

        self._ui.cmd_delete.clicked.connect(lambda status, x=my_id: self.delete_me.emit(x))
        self._ui.le_name.editingFinished.connect(self._new_name)

        self._ui.cmb_camera_type.addItems(['Dummy', 'LMScreen', 'Vimba'])
        self._ui.cmb_motor_type.addItems(['None', 'FSBT', 'Acromag'])

        if settings_node is not None:
            self._ui.le_name.setText(settings_node.get('name'))

            if 'enabled' in settings_node.keys():
                self._ui.chk_enabled.setChecked(strtobool(settings_node.get('enabled')))
            else:
                self._ui.chk_enabled.setChecked(True)

            camera_type = settings_node.get('proxy')
            if camera_type == 'DummyProxy':
                refresh_combo_box(self._ui.cmb_camera_type, 'Dummy')
            elif camera_type == 'TangoTineProxy':
                refresh_combo_box(self._ui.cmb_camera_type, 'LMScreen')
                self._ui.fr_tango.setVisible(True)
            elif camera_type == 'VimbaProxy':
                refresh_combo_box(self._ui.cmb_camera_type, 'Vimba')
                self._ui.fr_tango.setVisible(True)
            else:
                raise RuntimeError('Unknown type')

            if 'tango_server' in settings_node.keys():
                self._ui.le_tango_server.setText(settings_node.get('tango_server'))
            if 'settings_server' in settings_node.keys():
                self._ui.le_settings_server.setText(settings_node.get('settings_server'))
            if 'roi_server' in settings_node.keys():
                self._ui.le_roi_server.setText(settings_node.get('roi_server'))

            motor_type = settings_node.get('motor_type')
            if motor_type == 'FSBT':
                refresh_combo_box(self._ui.cmb_motor_type, 'FSBT')
                self._ui.le_fsbt_motor_name.setText(settings_node.get('motor_name'))
                self._ui.le_fsbt_host.setText(settings_node.get('motor_host'))
                self._ui.le_fsbt_port.setText(settings_node.get('motor_port'))
                self._ui.fr_fsbt.setVisible(True)
            elif motor_type == 'Acromag':
                refresh_combo_box(self._ui.cmb_motor_type, 'Acromag')
                self._ui.le_acromag_server.setText(settings_node.get('valve_tango_server'))
                self._ui.le_acromag_valve.setText(settings_node.get('valve_channel'))
                self._ui.fr_acromag.setVisible(True)
            else:
                refresh_combo_box(self._ui.cmb_motor_type, 'None')

            if 'flip_vertical' in settings_node.keys():
                self._ui.chk_flip_v.setChecked(strtobool(settings_node.get('flip_vertical')))
            if 'flip_horizontal' in settings_node.keys():
                self._ui.chk_flip_h.setChecked(strtobool(settings_node.get('flip_horizontal')))
            if 'rotate' in settings_node.keys():
                self._ui.sp_rotate.setValue(int(settings_node.get('rotate')))

            pass

    # ----------------------------------------------------------------------
    def _switch_camera_type(self, mode):
        self._ui.fr_tango.setVisible(False)
        self._ui.fr_roi.setVisible(False)
        self._ui.fr_settings.setVisible(False)
        if mode == 'LMScreen':
            self._ui.fr_tango.setVisible(True)
        elif mode == 'Vimba':
            self._ui.fr_tango.setVisible(True)

    # ----------------------------------------------------------------------
    def _switch_motor_type(self, mode):
        self._ui.fr_acromag.setVisible(False)
        self._ui.fr_fsbt.setVisible(False)

        if mode == 'FSBT':
            self._ui.fr_fsbt.setVisible(True)
        elif mode == 'Acromag':
            self._ui.fr_acromag.setVisible(True)

    # ----------------------------------------------------------------------
    def get_data(self):
        data_to_save = [('name', self._ui.le_name.text()),
                        ('enabled', str(self._ui.chk_enabled.isChecked()))]

        if self._ui.cmb_camera_type.currentText() == 'LMScreen':
            type = 'TangoTineProxy'
        elif self._ui.cmb_camera_type.currentText() == 'Vimba':
            type = 'VimbaProxy'
        else:
            type = 'DummyProxy'

        data_to_save.append(('proxy', type))

        if self._ui.cmb_motor_type.currentText() == 'FSBT':
            data_to_save.append(('motor_type', 'FSBT'))
            data_to_save.append(('motor_name', self._ui.le_fsbt_motor_name.text()))
            data_to_save.append(('motor_host', self._ui.le_fsbt_host.text()))
            data_to_save.append(('motor_port', self._ui.le_fsbt_port.text()))

        elif self._ui.cmb_motor_type.currentText() == 'Acromag':
            data_to_save.append(('motor_type', 'Acromag'))
            data_to_save.append(('valve_tango_server', self._ui.le_acromag_server.text()))
            data_to_save.append(('valve_channel', self._ui.le_acromag_valve.text()))
        else:
            data_to_save.append((('motor_type', 'none')),)

        if type == 'TangoTineProxy':
            data_to_save.append(('tango_server', self._ui.le_tango_server.text()))
        elif type == 'VimbaProxy':
            data_to_save.append(('tango_server', self._ui.le_tango_server.text()))

        data_to_save.append(('flip_vertical', str(self._ui.chk_flip_v.isChecked())))
        data_to_save.append(('flip_horizontal', str(self._ui.chk_flip_h.isChecked())))
        data_to_save.append(('rotate', str(self._ui.sp_rotate.value())))

        return data_to_save

    # ----------------------------------------------------------------------
    def _new_name(self):
        self.new_name.emit(self.my_id, self._ui.le_name.text())


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'uis/SettingsDialog.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_SettingsDialog(object):
    def setupUi(self, SettingsDialog):
        SettingsDialog.setObjectName("SettingsDialog")
        SettingsDialog.resize(629, 799)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(SettingsDialog)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.groupBox = QtWidgets.QGroupBox(SettingsDialog)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(self.groupBox)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.le_save_path = QtWidgets.QLineEdit(self.groupBox)
        self.le_save_path.setObjectName("le_save_path")
        self.horizontalLayout.addWidget(self.le_save_path)
        self.cmd_save_folder = QtWidgets.QPushButton(self.groupBox)
        self.cmd_save_folder.setObjectName("cmd_save_folder")
        self.horizontalLayout.addWidget(self.cmd_save_folder)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.groupBox_2 = QtWidgets.QGroupBox(self.groupBox)
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_3 = QtWidgets.QLabel(self.groupBox_2)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_2.addWidget(self.label_3)
        self.cmd_roi_frame_color = QtWidgets.QPushButton(self.groupBox_2)
        self.cmd_roi_frame_color.setText("")
        self.cmd_roi_frame_color.setObjectName("cmd_roi_frame_color")
        self.horizontalLayout_2.addWidget(self.cmd_roi_frame_color)
        self.label_4 = QtWidgets.QLabel(self.groupBox_2)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_2.addWidget(self.label_4)
        self.cmd_roi_label_font = QtWidgets.QPushButton(self.groupBox_2)
        self.cmd_roi_label_font.setText("")
        self.cmd_roi_label_font.setObjectName("cmd_roi_label_font")
        self.horizontalLayout_2.addWidget(self.cmd_roi_label_font)
        self.label_14 = QtWidgets.QLabel(self.groupBox_2)
        self.label_14.setObjectName("label_14")
        self.horizontalLayout_2.addWidget(self.label_14)
        self.cmd_roi_label_font_color = QtWidgets.QPushButton(self.groupBox_2)
        self.cmd_roi_label_font_color.setText("")
        self.cmd_roi_label_font_color.setObjectName("cmd_roi_label_font_color")
        self.horizontalLayout_2.addWidget(self.cmd_roi_label_font_color)
        self.label_13 = QtWidgets.QLabel(self.groupBox_2)
        self.label_13.setObjectName("label_13")
        self.horizontalLayout_2.addWidget(self.label_13)
        self.cmd_roi_bkg_color = QtWidgets.QPushButton(self.groupBox_2)
        self.cmd_roi_bkg_color.setText("")
        self.cmd_roi_bkg_color.setObjectName("cmd_roi_bkg_color")
        self.horizontalLayout_2.addWidget(self.cmd_roi_bkg_color)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_6 = QtWidgets.QLabel(self.groupBox_2)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_3.addWidget(self.label_6)
        self.cmd_marker_color = QtWidgets.QPushButton(self.groupBox_2)
        self.cmd_marker_color.setText("")
        self.cmd_marker_color.setObjectName("cmd_marker_color")
        self.horizontalLayout_3.addWidget(self.cmd_marker_color)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_5 = QtWidgets.QLabel(self.groupBox_2)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_4.addWidget(self.label_5)
        self.cmd_title_font = QtWidgets.QPushButton(self.groupBox_2)
        self.cmd_title_font.setText("")
        self.cmd_title_font.setObjectName("cmd_title_font")
        self.horizontalLayout_4.addWidget(self.cmd_title_font)
        self.label_15 = QtWidgets.QLabel(self.groupBox_2)
        self.label_15.setObjectName("label_15")
        self.horizontalLayout_4.addWidget(self.label_15)
        self.cmd_title_font_color = QtWidgets.QPushButton(self.groupBox_2)
        self.cmd_title_font_color.setText("")
        self.cmd_title_font_color.setObjectName("cmd_title_font_color")
        self.horizontalLayout_4.addWidget(self.cmd_title_font_color)
        self.label_8 = QtWidgets.QLabel(self.groupBox_2)
        self.label_8.setObjectName("label_8")
        self.horizontalLayout_4.addWidget(self.label_8)
        self.cmd_title_bkg_color = QtWidgets.QPushButton(self.groupBox_2)
        self.cmd_title_bkg_color.setText("")
        self.cmd_title_bkg_color.setObjectName("cmd_title_bkg_color")
        self.horizontalLayout_4.addWidget(self.cmd_title_bkg_color)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem2)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.verticalLayout_2.addWidget(self.groupBox_2)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_7 = QtWidgets.QLabel(self.groupBox)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_5.addWidget(self.label_7)
        self.chk_roi_server_enable = QtWidgets.QCheckBox(self.groupBox)
        self.chk_roi_server_enable.setObjectName("chk_roi_server_enable")
        self.horizontalLayout_5.addWidget(self.chk_roi_server_enable)
        self.label_9 = QtWidgets.QLabel(self.groupBox)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_5.addWidget(self.label_9)
        self.le_roi_server_host = QtWidgets.QLineEdit(self.groupBox)
        self.le_roi_server_host.setObjectName("le_roi_server_host")
        self.horizontalLayout_5.addWidget(self.le_roi_server_host)
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_5.addWidget(self.label_2)
        self.le_roi_server_port = QtWidgets.QLineEdit(self.groupBox)
        self.le_roi_server_port.setObjectName("le_roi_server_port")
        self.horizontalLayout_5.addWidget(self.le_roi_server_port)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem3)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label_10 = QtWidgets.QLabel(self.groupBox)
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_6.addWidget(self.label_10)
        self.label_11 = QtWidgets.QLabel(self.groupBox)
        self.label_11.setObjectName("label_11")
        self.horizontalLayout_6.addWidget(self.label_11)
        self.dsb_cross_size = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.dsb_cross_size.setObjectName("dsb_cross_size")
        self.horizontalLayout_6.addWidget(self.dsb_cross_size)
        self.label_12 = QtWidgets.QLabel(self.groupBox)
        self.label_12.setObjectName("label_12")
        self.horizontalLayout_6.addWidget(self.label_12)
        self.dsb_circle_size = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.dsb_circle_size.setObjectName("dsb_circle_size")
        self.horizontalLayout_6.addWidget(self.dsb_circle_size)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem4)
        self.verticalLayout_2.addLayout(self.horizontalLayout_6)
        self.verticalLayout_3.addWidget(self.groupBox)
        self.tb_cameras = QtWidgets.QTabWidget(SettingsDialog)
        self.tb_cameras.setObjectName("tb_cameras")
        self.verticalLayout_3.addWidget(self.tb_cameras)
        self.buttonBox = QtWidgets.QDialogButtonBox(SettingsDialog)
        self.buttonBox.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout_3.addWidget(self.buttonBox)

        self.retranslateUi(SettingsDialog)
        self.tb_cameras.setCurrentIndex(-1)
        self.buttonBox.accepted.connect(SettingsDialog.accept)
        self.buttonBox.rejected.connect(SettingsDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(SettingsDialog)

    def retranslateUi(self, SettingsDialog):
        _translate = QtCore.QCoreApplication.translate
        SettingsDialog.setWindowTitle(_translate("SettingsDialog", "General settings"))
        self.groupBox.setTitle(_translate("SettingsDialog", "General settings"))
        self.label.setText(_translate("SettingsDialog", "Default save folder"))
        self.cmd_save_folder.setText(_translate("SettingsDialog", "Select folder"))
        self.groupBox_2.setTitle(_translate("SettingsDialog", "Colors"))
        self.label_3.setText(_translate("SettingsDialog", "ROI frame color"))
        self.label_4.setText(_translate("SettingsDialog", "ROI label font"))
        self.label_14.setText(_translate("SettingsDialog", "ROI label font color"))
        self.label_13.setText(_translate("SettingsDialog", "ROI label background"))
        self.label_6.setText(_translate("SettingsDialog", "Marker"))
        self.label_5.setText(_translate("SettingsDialog", "Title font"))
        self.label_15.setText(_translate("SettingsDialog", "Title font color"))
        self.label_8.setText(_translate("SettingsDialog", "Title background"))
        self.label_7.setText(_translate("SettingsDialog", "ROI server"))
        self.chk_roi_server_enable.setText(_translate("SettingsDialog", "enable"))
        self.label_9.setText(_translate("SettingsDialog", "host"))
        self.label_2.setText(_translate("SettingsDialog", "post"))
        self.label_10.setText(_translate("SettingsDialog", "Center search:"))
        self.label_11.setText(_translate("SettingsDialog", "cross size"))
        self.label_12.setText(_translate("SettingsDialog", "circle size"))




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------
# Author:        yury.matveev@desy.de
# ----------------------------------------------------------------------

"""Parse settings given in xml files
"""

import xml.etree.cElementTree as ET


# ----------------------------------------------------------------------
class XmlSettings(object):

    # ----------------------------------------------------------------------
    def __init__(self, file_name):
        self.file_name = file_name

    # ----------------------------------------------------------------------
    def set_options(self, general_settings, cameras_settings):

        et_tree = ET.parse(self.file_name)
        root = et_tree.getroot()
        for node, values in general_settings:
            try:
                node = self.node(node, root)
            except:
                node = self.make_node(root, node)

            for attribute, value in values:
                node.set(attribute, str(value))

        for el in root.findall('camera'):
            root.remove(el)

        el = None
        for camera in cameras_settings:
            el = ET.SubElement(root, 'camera', dict((key, value) for (key, value) in camera))
            el.tail = '\n\n\t'

        if el is not None:
            el.tail = '\n\n'

        et_tree.write(self.file_name)

    # ----------------------------------------------------------------------
    def option(self, node_path, attribute):
        """Retrieve option's value from a config file.
        """

        return self.node(node_path).get(attribute)

    # ----------------------------------------------------------------------
    def get_nodes(self, node_name, node_path=None):
        """
        """

        return self.node(node_path).findall(node_name)

    # ----------------------------------------------------------------------
    def make_node(self, root, node_path):
        for node in node_path.split("/"):
            if root.find(node) is not None:
                root = root.find(node)
            else:
                root = ET.SubElement(root, node)

        return root

    # ----------------------------------------------------------------------
    def node(self, node_path=None, root=None):
        """
        """
        if root is None:
            root = ET.parse(self.file_name).getroot()

        if node_path is not None:
            for node in node_path.split("/"):
                if root.find(node) is not None:
                    root = root.find(node)
                else:
                    raise RuntimeError('Wrong xml path')

        return root

    # ----------------------------------------------------------------------
    def has_node(self, node_path):
        try:
            self.node(node_path)
            return True
        except:
            return False



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'uis/MarkersROIs.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MarkersROIs(object):
    def setupUi(self, MarkersROIs):
        MarkersROIs.setObjectName("MarkersROIs")
        MarkersROIs.resize(457, 770)
        self.verticalLayout = QtWidgets.QVBoxLayout(MarkersROIs)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox_2 = QtWidgets.QGroupBox(MarkersROIs)
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.but_add_marker = QtWidgets.QPushButton(self.groupBox_2)
        self.but_add_marker.setObjectName("but_add_marker")
        self.horizontalLayout.addWidget(self.but_add_marker)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.layout_markers = QtWidgets.QWidget(self.groupBox_2)
        self.layout_markers.setMinimumSize(QtCore.QSize(0, 0))
        self.layout_markers.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.layout_markers.setObjectName("layout_markers")
        self.verticalLayout_2.addWidget(self.layout_markers)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem1)
        self.verticalLayout.addWidget(self.groupBox_2)
        self.groupBox_3 = QtWidgets.QGroupBox(MarkersROIs)
        self.groupBox_3.setObjectName("groupBox_3")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.groupBox_3)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_14.addItem(spacerItem2)
        self.but_add_roi = QtWidgets.QPushButton(self.groupBox_3)
        self.but_add_roi.setObjectName("but_add_roi")
        self.horizontalLayout_14.addWidget(self.but_add_roi)
        self.verticalLayout_8.addLayout(self.horizontalLayout_14)
        self.tb_rois = QtWidgets.QTabWidget(self.groupBox_3)
        self.tb_rois.setObjectName("tb_rois")
        self.verticalLayout_8.addWidget(self.tb_rois)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_8.addItem(spacerItem3)
        self.verticalLayout.addWidget(self.groupBox_3)
        self.gb_sardana = QtWidgets.QGroupBox(MarkersROIs)
        self.gb_sardana.setObjectName("gb_sardana")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.gb_sardana)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.lb_screen_status_3 = QtWidgets.QLabel(self.gb_sardana)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.lb_screen_status_3.setFont(font)
        self.lb_screen_status_3.setObjectName("lb_screen_status_3")
        self.horizontalLayout_3.addWidget(self.lb_screen_status_3)
        self.cmb_roi_as_counter = QtWidgets.QComboBox(self.gb_sardana)
        self.cmb_roi_as_counter.setObjectName("cmb_roi_as_counter")
        self.horizontalLayout_3.addWidget(self.cmb_roi_as_counter)
        self.lb_screen_status_2 = QtWidgets.QLabel(self.gb_sardana)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.lb_screen_status_2.setFont(font)
        self.lb_screen_status_2.setObjectName("lb_screen_status_2")
        self.horizontalLayout_3.addWidget(self.lb_screen_status_2)
        self.cb_counter = QtWidgets.QComboBox(self.gb_sardana)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cb_counter.sizePolicy().hasHeightForWidth())
        self.cb_counter.setSizePolicy(sizePolicy)
        self.cb_counter.setMinimumSize(QtCore.QSize(0, 0))
        self.cb_counter.setObjectName("cb_counter")
        self.cb_counter.addItem("")
        self.cb_counter.addItem("")
        self.cb_counter.addItem("")
        self.cb_counter.addItem("")
        self.cb_counter.addItem("")
        self.cb_counter.addItem("")
        self.cb_counter.addItem("")
        self.cb_counter.addItem("")
        self.horizontalLayout_3.addWidget(self.cb_counter)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem4)
        self.verticalLayout.addWidget(self.gb_sardana)
        spacerItem5 = QtWidgets.QSpacerItem(436, 224, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem5)

        self.retranslateUi(MarkersROIs)
        QtCore.QMetaObject.connectSlotsByName(MarkersROIs)

    def retranslateUi(self, MarkersROIs):
        _translate = QtCore.QCoreApplication.translate
        MarkersROIs.setWindowTitle(_translate("MarkersROIs", "Markers/ROIs"))
        self.groupBox_2.setTitle(_translate("MarkersROIs", "Markers"))
        self.but_add_marker.setText(_translate("MarkersROIs", "Add Marker"))
        self.groupBox_3.setTitle(_translate("MarkersROIs", "ROIs"))
        self.but_add_roi.setText(_translate("MarkersROIs", "Add ROI"))
        self.gb_sardana.setTitle(_translate("MarkersROIs", "Sardana \"counter\" behavior"))
        self.lb_screen_status_3.setText(_translate("MarkersROIs", "Source ROI:"))
        self.lb_screen_status_2.setText(_translate("MarkersROIs", "Statistics value as counter:"))
        self.cb_counter.setItemText(0, _translate("MarkersROIs", "sum"))
        self.cb_counter.setItemText(1, _translate("MarkersROIs", "max_i"))
        self.cb_counter.setItemText(2, _translate("MarkersROIs", "max_x"))
        self.cb_counter.setItemText(3, _translate("MarkersROIs", "max_y"))
        self.cb_counter.setItemText(4, _translate("MarkersROIs", "com_x"))
        self.cb_counter.setItemText(5, _translate("MarkersROIs", "com_y"))
        self.cb_counter.setItemText(6, _translate("MarkersROIs", "fwhm_x"))
        self.cb_counter.setItemText(7, _translate("MarkersROIs", "fwhm_y"))




# ----------------------------------------------------------------------
# Author:        yury.matveev@desy.de
# ----------------------------------------------------------------------

"""
"""

from PyQt5 import QtWidgets, QtCore
from petra_camera.gui.ROI_ui import Ui_Roi

# ----------------------------------------------------------------------
class ROI(QtWidgets.QWidget):

    delete_me = QtCore.pyqtSignal(int)
    repaint_roi = QtCore.pyqtSignal()

    # ----------------------------------------------------------------------
    def __init__(self, id, camera_device):

        super(ROI, self).__init__()

        self._ui = Ui_Roi()
        self._ui.setupUi(self)

        self.my_id = id
        self.camera_device = camera_device
        self._enable_me(camera_device.rois[id]['visible'])

        self._ui.but_delete.clicked.connect(lambda state, my_id=self.my_id: self.delete_me.emit(my_id))

        self._ui.chb_roi_enable.clicked.connect(self._enable_me)

        self._ui.chk_show_com.clicked.connect(lambda state, name='com': self._enable_cross(name))
        self._ui.chk_show_max.clicked.connect(lambda state, name='max': self._enable_cross(name))
        self._ui.chk_show_min.clicked.connect(lambda state, name='min': self._enable_cross(name))

        self._ui.sb_roi_bg.valueChanged.connect(lambda value: self.save_value('bg', value))
        self._ui.sb_roi_x.valueChanged.connect(lambda value: self.save_value('x', value))
        self._ui.sb_roi_y.valueChanged.connect(lambda value: self.save_value('y', value))
        self._ui.sb_roi_w.valueChanged.connect(lambda value: self.save_value('w', value))
        self._ui.sb_roi_h.valueChanged.connect(lambda value: self.save_value('h', value))

        self._ui.but_color.clicked.connect(self._pick_my_color)

        self.update_values()

    # ----------------------------------------------------------------------
    def _block_signals(self, flag):

        self._ui.chb_roi_enable.blockSignals(flag)
        self._ui.but_delete.blockSignals(flag)

        self._ui.chk_show_min.blockSignals(flag)
        self._ui.chk_show_com.blockSignals(flag)
        self._ui.chk_show_max.blockSignals(flag)

        self._ui.sb_roi_bg.blockSignals(flag)

        self._ui.sb_roi_h.blockSignals(flag)
        self._ui.sb_roi_w.blockSignals(flag)
        self._ui.sb_roi_x.blockSignals(flag)
        self._ui.sb_roi_y.blockSignals(flag)

    # ----------------------------------------------------------------------
    def save_value(self, param, value):

        self.camera_device.set_roi_value(self.my_id, param, value)
        self.camera_device.calculate_roi_statistics()

        self.repaint_roi.emit()

    # ----------------------------------------------------------------------
    def update_values(self):

        self._block_signals(True)
        for ui in ['max_x', 'max_y', 'max_v',
                   'min_x', 'min_y', 'min_v',
                   'com_x', 'com_y', 'com_v',
                   'fwhm_x', 'fwhm_y', 'sum']:
            if self.camera_device.rois[self.my_id]['visible']:
                try:
                    getattr(self._ui, 'le_{}'.format(ui)).setText('{}'.format(self.camera_device.rois_data[self.my_id][ui]))
                except:
                    getattr(self._ui, 'le_{}'.format(ui)).setText('')
            else:
                getattr(self._ui, 'le_{}'.format(ui)).setText('')

        for ui in ['x', 'y', 'w', 'h', 'bg']:
            getattr(self._ui, 'sb_roi_{}'.format(ui)).setValue(self.camera_device.rois[self.my_id][ui])

        for ui in ['max', 'min', 'com']:
            getattr(self._ui, 'chk_show_{}'.format(ui)).setChecked(self.camera_device.rois[self.my_id]['mark'] == ui)

        self._ui.chb_roi_enable.setChecked(self.camera_device.rois[self.my_id]['visible'])

        self._ui.but_color.setStyleSheet("QPushButton {background-color: " +
                                         f"{self.camera_device.rois[self.my_id]['color']}" + ";}")

        self._block_signals(False)

    # ----------------------------------------------------------------------
    def _pick_my_color(self):
        color = QtWidgets.QColorDialog.getColor()

        if color.isValid():
            self.camera_device.set_roi_value(self.my_id, 'color', color.name())

            self.repaint_roi.emit()

    # ----------------------------------------------------------------------
    def _enable_me(self, state):
        self._ui.chk_show_min.setEnabled(state)
        self._ui.chk_show_com.setEnabled(state)
        self._ui.chk_show_max.setEnabled(state)

        self._ui.sb_roi_bg.setEnabled(state)

        self._ui.sb_roi_h.setEnabled(state)
        self._ui.sb_roi_w.setEnabled(state)
        self._ui.sb_roi_x.setEnabled(state)
        self._ui.sb_roi_y.setEnabled(state)

        self.camera_device.set_roi_value(self.my_id, 'visible', state)

        self.repaint_roi.emit()

    # ----------------------------------------------------------------------
    def _enable_cross(self, name):
        if self.camera_device.rois[self.my_id]['mark'] == name:
            name = ''

        for ui in ['max', 'min', 'com']:
            getattr(self._ui, 'chk_show_{}'.format(ui)).setChecked(name == ui)

        self.camera_device.set_roi_value(self.my_id, 'mark', name)

        self.repaint_roi.emit()



#!/usr/bin/env python

# ----------------------------------------------------------------------
# Author:        sebastian.piec@desy.de & yury.matveev@desy.de
# Last modified: 2017, December 12
# ----------------------------------------------------------------------

"""Auxiliary functions used in many different contexts.
"""

import datetime
import logging
import math
import os

import numpy as np


# ----------------------------------------------------------------------
def refresh_combo_box(comboBox, text):
    """Auxiliary function refreshing combo box with a given text.
    """
    idx = comboBox.findText(text)
    comboBox.blockSignals(True)
    if idx != -1:
        comboBox.setCurrentIndex(idx)
        comboBox.blockSignals(False)
        return True
    else:
        comboBox.setCurrentIndex(0)
        comboBox.blockSignals(False)
        return False


# ----------------------------------------------------------------------
def rotate(origin, point, angle):
    """
    Rotate a point counterclockwise by a given angle around a given origin.

    The angle should be given in radians.
    """
    ox, oy = origin
    px, py = point

    qx = ox + math.cos(angle) * (px - ox) - math.sin(angle) * (py - oy)
    qy = oy + math.sin(angle) * (px - ox) + math.cos(angle) * (py - oy)
    return qx, qy

# ----------------------------------------------------------------------
def FWHM(data):
    """
    simple calculator of peak FWHM
    :param data:
    :return:
    """
    try:
        half_max = (np.amax(data) - np.amin(data)) / 2

        diff = np.sign(data - half_max)
        left_idx = np.where(diff > 0)[0][0]
        right_idx = np.where(diff > 0)[0][-1]
        return right_idx - left_idx  # return the difference (full width)
    except:
        return 0



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'uis/CameraSettings.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_CameraSettings(object):
    def setupUi(self, CameraSettings):
        CameraSettings.setObjectName("CameraSettings")
        CameraSettings.resize(557, 484)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(CameraSettings)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(CameraSettings)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.cmb_camera_type = QtWidgets.QComboBox(CameraSettings)
        self.cmb_camera_type.setObjectName("cmb_camera_type")
        self.horizontalLayout.addWidget(self.cmb_camera_type)
        self.label_12 = QtWidgets.QLabel(CameraSettings)
        self.label_12.setObjectName("label_12")
        self.horizontalLayout.addWidget(self.label_12)
        self.le_name = QtWidgets.QLineEdit(CameraSettings)
        self.le_name.setObjectName("le_name")
        self.horizontalLayout.addWidget(self.le_name)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.chk_enabled = QtWidgets.QCheckBox(CameraSettings)
        self.chk_enabled.setObjectName("chk_enabled")
        self.horizontalLayout.addWidget(self.chk_enabled)
        self.cmd_delete = QtWidgets.QPushButton(CameraSettings)
        self.cmd_delete.setObjectName("cmd_delete")
        self.horizontalLayout.addWidget(self.cmd_delete)
        self.verticalLayout_5.addLayout(self.horizontalLayout)
        self.fr_tango = QtWidgets.QFrame(CameraSettings)
        self.fr_tango.setObjectName("fr_tango")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.fr_tango)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_2 = QtWidgets.QLabel(self.fr_tango)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_4.addWidget(self.label_2)
        self.le_tango_server = QtWidgets.QLineEdit(self.fr_tango)
        self.le_tango_server.setObjectName("le_tango_server")
        self.horizontalLayout_4.addWidget(self.le_tango_server)
        self.verticalLayout_5.addWidget(self.fr_tango)
        self.fr_roi = QtWidgets.QFrame(CameraSettings)
        self.fr_roi.setObjectName("fr_roi")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(self.fr_roi)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_3 = QtWidgets.QLabel(self.fr_roi)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_5.addWidget(self.label_3)
        self.le_roi_server = QtWidgets.QLineEdit(self.fr_roi)
        self.le_roi_server.setObjectName("le_roi_server")
        self.horizontalLayout_5.addWidget(self.le_roi_server)
        self.verticalLayout_5.addWidget(self.fr_roi)
        self.fr_settings = QtWidgets.QFrame(CameraSettings)
        self.fr_settings.setObjectName("fr_settings")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout(self.fr_settings)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label_4 = QtWidgets.QLabel(self.fr_settings)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_6.addWidget(self.label_4)
        self.le_settings_server = QtWidgets.QLineEdit(self.fr_settings)
        self.le_settings_server.setObjectName("le_settings_server")
        self.horizontalLayout_6.addWidget(self.le_settings_server)
        self.verticalLayout_5.addWidget(self.fr_settings)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_5 = QtWidgets.QLabel(CameraSettings)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_2.addWidget(self.label_5)
        self.cmb_motor_type = QtWidgets.QComboBox(CameraSettings)
        self.cmb_motor_type.setObjectName("cmb_motor_type")
        self.horizontalLayout_2.addWidget(self.cmb_motor_type)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.verticalLayout_5.addLayout(self.horizontalLayout_2)
        self.fr_motor = QtWidgets.QFrame(CameraSettings)
        self.fr_motor.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.fr_motor.setFrameShadow(QtWidgets.QFrame.Raised)
        self.fr_motor.setObjectName("fr_motor")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.fr_motor)
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_4.setSpacing(0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.fr_fsbt = QtWidgets.QFrame(self.fr_motor)
        self.fr_fsbt.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.fr_fsbt.setFrameShadow(QtWidgets.QFrame.Raised)
        self.fr_fsbt.setObjectName("fr_fsbt")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.fr_fsbt)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_7 = QtWidgets.QLabel(self.fr_fsbt)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_7.addWidget(self.label_7)
        self.le_fsbt_host = QtWidgets.QLineEdit(self.fr_fsbt)
        self.le_fsbt_host.setObjectName("le_fsbt_host")
        self.horizontalLayout_7.addWidget(self.le_fsbt_host)
        self.verticalLayout_2.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.label_8 = QtWidgets.QLabel(self.fr_fsbt)
        self.label_8.setObjectName("label_8")
        self.horizontalLayout_8.addWidget(self.label_8)
        self.le_fsbt_port = QtWidgets.QLineEdit(self.fr_fsbt)
        self.le_fsbt_port.setObjectName("le_fsbt_port")
        self.horizontalLayout_8.addWidget(self.le_fsbt_port)
        self.verticalLayout_2.addLayout(self.horizontalLayout_8)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.label_9 = QtWidgets.QLabel(self.fr_fsbt)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_9.addWidget(self.label_9)
        self.le_fsbt_motor_name = QtWidgets.QLineEdit(self.fr_fsbt)
        self.le_fsbt_motor_name.setObjectName("le_fsbt_motor_name")
        self.horizontalLayout_9.addWidget(self.le_fsbt_motor_name)
        self.verticalLayout_2.addLayout(self.horizontalLayout_9)
        self.verticalLayout_4.addWidget(self.fr_fsbt)
        self.fr_acromag = QtWidgets.QFrame(self.fr_motor)
        self.fr_acromag.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.fr_acromag.setFrameShadow(QtWidgets.QFrame.Raised)
        self.fr_acromag.setObjectName("fr_acromag")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.fr_acromag)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.label_10 = QtWidgets.QLabel(self.fr_acromag)
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_10.addWidget(self.label_10)
        self.le_acromag_server = QtWidgets.QLineEdit(self.fr_acromag)
        self.le_acromag_server.setObjectName("le_acromag_server")
        self.horizontalLayout_10.addWidget(self.le_acromag_server)
        self.verticalLayout_3.addLayout(self.horizontalLayout_10)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.label_11 = QtWidgets.QLabel(self.fr_acromag)
        self.label_11.setObjectName("label_11")
        self.horizontalLayout_11.addWidget(self.label_11)
        self.le_acromag_valve = QtWidgets.QLineEdit(self.fr_acromag)
        self.le_acromag_valve.setObjectName("le_acromag_valve")
        self.horizontalLayout_11.addWidget(self.le_acromag_valve)
        self.verticalLayout_3.addLayout(self.horizontalLayout_11)
        self.verticalLayout_4.addWidget(self.fr_acromag)
        self.verticalLayout_5.addWidget(self.fr_motor)
        self.groupBox = QtWidgets.QGroupBox(CameraSettings)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.chk_flip_v = QtWidgets.QCheckBox(self.groupBox)
        self.chk_flip_v.setObjectName("chk_flip_v")
        self.verticalLayout.addWidget(self.chk_flip_v)
        self.chk_flip_h = QtWidgets.QCheckBox(self.groupBox)
        self.chk_flip_h.setObjectName("chk_flip_h")
        self.verticalLayout.addWidget(self.chk_flip_h)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_6 = QtWidgets.QLabel(self.groupBox)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_3.addWidget(self.label_6)
        self.sp_rotate = QtWidgets.QSpinBox(self.groupBox)
        self.sp_rotate.setMaximum(3)
        self.sp_rotate.setObjectName("sp_rotate")
        self.horizontalLayout_3.addWidget(self.sp_rotate)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem2)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.verticalLayout_5.addWidget(self.groupBox)
        spacerItem3 = QtWidgets.QSpacerItem(20, 70, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_5.addItem(spacerItem3)

        self.retranslateUi(CameraSettings)
        QtCore.QMetaObject.connectSlotsByName(CameraSettings)

    def retranslateUi(self, CameraSettings):
        _translate = QtCore.QCoreApplication.translate
        CameraSettings.setWindowTitle(_translate("CameraSettings", "CameraSettings"))
        self.label.setText(_translate("CameraSettings", "Camera type"))
        self.label_12.setText(_translate("CameraSettings", "Name:"))
        self.chk_enabled.setText(_translate("CameraSettings", "Enabled"))
        self.cmd_delete.setText(_translate("CameraSettings", "Delete"))
        self.label_2.setText(_translate("CameraSettings", "Tango server"))
        self.label_3.setText(_translate("CameraSettings", "ROI server"))
        self.label_4.setText(_translate("CameraSettings", "Settings server"))
        self.label_5.setText(_translate("CameraSettings", "Screen motor type"))
        self.label_7.setText(_translate("CameraSettings", "FSBT server host"))
        self.label_8.setText(_translate("CameraSettings", "FSBT server post"))
        self.label_9.setText(_translate("CameraSettings", "FSBT motor name"))
        self.label_10.setText(_translate("CameraSettings", "Valve tango server"))
        self.label_11.setText(_translate("CameraSettings", "Valve channel"))
        self.groupBox.setTitle(_translate("CameraSettings", "Picture settings"))
        self.chk_flip_v.setText(_translate("CameraSettings", "Flip vertical"))
        self.chk_flip_h.setText(_translate("CameraSettings", "Flip horizontal"))
        self.label_6.setText(_translate("CameraSettings", "Rotate in 90 deg step"))




#!/usr/bin/env python

# ----------------------------------------------------------------------
# Author:        yury.matveyev@desy.de
# ----------------------------------------------------------------------

"""
wrapper for threading.Thread, that collects error from thread and put it into Queue
"""

import threading
import sys
import traceback


class ExcThread(threading.Thread):

    # ----------------------------------------------------------------------
    def __init__(self, target, thread_name, error_bucket,  *args):
        self.ret = None
        threading.Thread.__init__(self, target=target, name=thread_name, args=args)
        self._stop_event = threading.Event()
        self.bucket = error_bucket

    # ----------------------------------------------------------------------
    def run(self):
        try:
            if hasattr(self, '_Thread__target'):
                self.ret = self._Thread__target(*self._Thread__args, **self._Thread__kwargs)
            else:
                self.ret = self._target(*self._args, **self._kwargs)

        except Exception as exp:
            print('exception caught in propagating thread:\n{}'.format(exp))
            traceback.print_tb(sys.exc_info()[2])
            self.bucket.put(sys.exc_info())

    # ----------------------------------------------------------------------
    def stop(self):
        self._stop_event.set()

    # ----------------------------------------------------------------------
    def stopped(self):
        return self._stop_event.isSet()


# Created by matveyev at 19.08.2021

import logging

from PyQt5 import QtWidgets, QtCore

from petra_camera.utils.functions import refresh_combo_box
from petra_camera.widgets.base_widget import BaseWidget
from petra_camera.widgets.marker import Marker
from petra_camera.widgets.roi_widget import ROI
from petra_camera.main_window import APP_NAME

from petra_camera.gui.MarkersROIs_ui import Ui_MarkersROIs

WIDGET_NAME = 'MarkersROIs'

logger = logging.getLogger(APP_NAME)


# ----------------------------------------------------------------------
class MarkersROIsWidget(BaseWidget):

    add_remove_roi = QtCore.pyqtSignal()
    repaint_roi = QtCore.pyqtSignal()

    add_remove_marker = QtCore.pyqtSignal()
    repaint_marker = QtCore.pyqtSignal()

    # ----------------------------------------------------------------------
    def __init__(self, parent):

        super(MarkersROIsWidget, self).__init__(parent)

        self._ui = Ui_MarkersROIs()
        self._ui.setupUi(self)

        self._ui.gb_sardana.setVisible(self._camera_device.has_counter())

        self._marker_grid = QtWidgets.QGridLayout(self._ui.layout_markers)
        self._markers_widgets = []

        self._rois, self._markers, self._statistics = None, None, None

        self._reload_rois()
        self._update_marker_layout()

        self._ui.but_add_marker.clicked.connect(self._add_marker)
        self._ui.but_add_roi.clicked.connect(self._add_roi)

        self._ui.cb_counter.currentTextChanged.connect(lambda text: self._camera_device.set_counter(text))
        self._ui.cmb_roi_as_counter.currentIndexChanged.connect(lambda ind: self._camera_device.set_counter_roi(ind))

    # ----------------------------------------------------------------------
    def update_roi_statistics(self):

        for idx in range(self._ui.tb_rois.count()):
            self._ui.tb_rois.widget(idx).update_values()

        for widget in self._markers_widgets:
            widget.update_values()

        if not self._ui.cb_counter.hasFocus():
            counter_name = self._camera_device.get_counter()
            if counter_name == '':
                counter_name = 'sum'
                self._camera_device.set_counter(counter_name)
            refresh_combo_box(self._ui.cb_counter, counter_name)

    # ----------------------------------------------------------------------
    def _add_roi_widget(self, index):
        widget = ROI(index, self._camera_device)
        widget.delete_me.connect(self._delete_roi)
        widget.repaint_roi.connect(lambda: self.repaint_roi.emit())

        self._ui.tb_rois.insertTab(index, widget, 'ROI_{}'.format(index+1))
        self._ui.tb_rois.setCurrentWidget(widget)

    # ----------------------------------------------------------------------
    def _add_roi(self):
        self._camera_device.add_roi()
        self._add_roi_widget(self._ui.tb_rois.count())
        self.add_remove_roi.emit()

        self._refresh_roi_as_counter_cmb()
        self._ui.tb_rois.setVisible(True)

    # ----------------------------------------------------------------------
    def _delete_roi(self, idx):
        self._ui.tb_rois.removeTab(idx)

        self._camera_device.delete_roi(idx)
        self.add_remove_roi.emit()

        self._ui.tb_rois.setVisible(self._camera_device.num_roi())
        self._refresh_roi_as_counter_cmb()

    # ----------------------------------------------------------------------
    def _refresh_roi_as_counter_cmb(self):
        self._ui.cmb_roi_as_counter.clear()
        if self._camera_device.num_roi():
            self._ui.cmb_roi_as_counter.setEnabled(True)
            for ind in range(self._camera_device.num_roi()):
                self._ui.cmb_roi_as_counter.addItem('ROI {}'.format(ind + 1))
            if self._camera_device.get_counter_roi() < self._camera_device.num_roi():
                self._ui.cmb_roi_as_counter.setCurrentIndex(self._camera_device.get_counter_roi())
            else:
                self._ui.cmb_roi_as_counter.setCurrentIndex(self._camera_device.num_roi()-1)
        else:
            self._ui.cmb_roi_as_counter.setEnabled(False)

    # ----------------------------------------------------------------------
    def _reload_rois(self):
        for ind in range(self._ui.tb_rois.count())[::-1]:
            self._ui.tb_rois.removeTab(ind)

        if len(self._camera_device.rois):
            self._ui.tb_rois.setVisible(True)
            for ind in range(len(self._camera_device.rois)):
                self._add_roi_widget(ind)
        else:
            self._ui.tb_rois.setVisible(False)

        self._refresh_roi_as_counter_cmb()

    # ----------------------------------------------------------------------
    def _add_marker(self):
        self._camera_device.append_marker()
        self._update_marker_layout()
        self.add_remove_marker.emit()

    # ----------------------------------------------------------------------
    def _delete_marker(self, id):

        self._camera_device.delete_marker(id)
        self._update_marker_layout()
        self.add_remove_marker.emit()

    # ----------------------------------------------------------------------
    def _update_marker_layout(self):

        layout = self._ui.layout_markers.layout()
        for i in reversed(range(layout.count())):
            item = layout.itemAt(i)
            if item:
                w = layout.itemAt(i).widget()
                if w:
                    layout.removeWidget(w)
                    w.setVisible(False)

        self._markers_widgets = []
        for ind in range(len(self._camera_device.markers)):
            widget = Marker(ind, self._camera_device)
            widget.repaint_marker.connect(lambda: self.repaint_marker.emit())
            widget.delete_me.connect(self._delete_marker)
            layout.addWidget(widget)
            self._markers_widgets.append(widget)



# ----------------------------------------------------------------------
# Author:        yury.matveev@desy.de
# ----------------------------------------------------------------------


"""TCP/IP server exposing some summary statistics about the data frame.

"""

import json

import select
import traceback
import time

import logging
import socket

from queue import Queue
from PyQt5 import QtCore

from petra_camera.utils.propagating_thread import ExcThread

from petra_camera.main_window import APP_NAME

logger = logging.getLogger(APP_NAME)

# ----------------------------------------------------------------------
class RoiServer(QtCore.QObject):
    """
    """

    SOCKET_TIMEOUT = .5                            # [s]
    MAX_REQUEST_LEN = 256
    MAX_CLIENT_NUMBER = 32
    CMD_LIST = ["get_sum",]

    # ----------------------------------------------------------------------
    def __init__(self, host, port, cameras_list):

        super(RoiServer, self).__init__()

        self._camera_device = None
        self._cameras_list = None

        self.host = str(host)
        self.port = int(port)

        self._cameras_list = cameras_list

        self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self._socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)      # TODO
        self._socket.settimeout(self.SOCKET_TIMEOUT)
        self._socket.bind((self.host, self.port))

        self._connectionMap = []

        self._state = "idle"

        self._errorQueue = Queue()
        self._serverWorker = ExcThread(self.run, 'roiServer', self._errorQueue)
        
        logger.info("ROI server host {}, port {}".format(self.host,
                                                           self.port))

    # ----------------------------------------------------------------------
    def start(self):
        """
        """
        self._serverWorker.start()

    # ----------------------------------------------------------------------
    def run(self):
        """
        """
        self._state = "run"
        self._socket.listen(self.MAX_CLIENT_NUMBER)

        read_list = [self._socket]
        while not self._serverWorker.stopped():
            readable, writable, errored = select.select(read_list, [], [], 0.1)
            for s in readable:
                if s is self._socket:
                    connection, address = self._socket.accept()
                    connection.setblocking(False)
                    self._connectionMap.append([connection, address])
                    logger.info('New client added: {}'.format(address))

            for connection, address in self._connectionMap:
                try:
                    request = connection.recv(self.MAX_REQUEST_LEN).decode()
                    if request:
                        connection.sendall(self._processRequest(request).encode())
                    else:
                        logger.info('Client closed: {}'.format(address))
                        connection.close()
                        self._connectionMap.remove([connection, address])

                except socket.error as err:
                    if err.args[0] == 10035:
                        if self._serverWorker.stopped():
                            break
                    elif err.errno == 11:
                        time.sleep(0.1)
                    elif err.errno == 10054:
                        logger.info('Client closed: {}'.format(address))
                        connection.close()
                        self._connectionMap.remove([connection, address])
                    else:
                        pass

                except KillConnection as _:
                    logger.error(traceback.format_exc())
                    logger.info('Client closed: {}'.format(address))
                    connection.close()
                    self._connectionMap.remove([connection, address])

                except Exception as _:
                    logger.error(traceback.format_exc())
                    break

        self._socket.close()
        self._state = 'aborted'

    # ----------------------------------------------------------------------
    def stop(self):
        """
        """
        self._serverWorker.stop()
        while self._state != 'aborted':
            time.sleep(0.1)

        logger.debug('ROI server stopped')

    # ----------------------------------------------------------------------
    def _processRequest(self, request):
        """
        Returns:
            (str) response, e.g.: "ok;manipulator;x;12.0"
        """
        response = ''
        print(__file__, "processing request '{}'".format(request))

        tokens = request.split()

        try:
            if len(tokens)> 1:
                response = self._make_response("OK", getattr(self, tokens[0])(tokens[1:]))
            else:
                response = self._make_response("OK", getattr(self, tokens[0])())

        except Exception as err:
            response = self._make_response("err", 'request unknown')

        print(__file__, "response '{}'".format(response))

        return response

    # ----------------------------------------------------------------------
    def _make_response(self, flag, message):
        """
        """
        return "{};{}".format(flag, json.dumps(message))

    # ----------------------------------------------------------------------
    def get_list_of_commands(self):

        return self.CMD_LIST

    # ----------------------------------------------------------------------
    def get_sum(self):

        return self._camera_device.get_active_roi_value('sum')

# ----------------------------------------------------------------------
class KillConnection(Exception):
    pass




# ----------------------------------------------------------------------
# Author:        yury.matveev@desy.de
# ----------------------------------------------------------------------

"""Dummy 2D data generator.
"""

import time
import numpy as np
import logging

from threading import Thread

from petra_camera.devices.base_camera import BaseCamera
from petra_camera.main_window import APP_NAME

logger = logging.getLogger(APP_NAME)


# ----------------------------------------------------------------------
class DummyProxy(BaseCamera):
    """
    """
    FRAME_W = 500
    FRAME_H = 500

    NOISE = 10

    _settings_map = {'max_width': ('self', 'FRAME_W'),
                     'max_height': ('self', 'FRAME_H')}

    visible_layouts = ('FPS', 'exposure')

    # ----------------------------------------------------------------------
    def __init__(self, settings):
        super(DummyProxy, self).__init__(settings)

        x, y = np.meshgrid(np.linspace(-4, 4, self.FRAME_H),
                           np.linspace(-4, 4, self.FRAME_W))
        x += 0.0
        y += -1.0

        mean, sigma = 0, 0.2
        self._baseData = 100 * np.exp(-((np.sqrt(x * x + y * y * 4) - mean) ** 4 / (2.0 * sigma ** 2)))
        self._data = self._baseData

        self.error_flag = False
        self.error_msg = ''

        self._fps = self.get_settings('FPS', int)
        if self._fps == 0:
            self._fps = 25

        self._generator_thread = Thread(target=self._generator)
        self._generate = False
        self._run = True
        self._new_frame_thead = Thread(target=self._new_frame)

        self._generator_thread.start()
        self._new_frame_thead.start()

        self._generator_thread_working = True
        self._new_frame_thead_working = True

    # ----------------------------------------------------------------------
    def close_camera(self):

        self._run = False
        while self._generator_thread_working or self._new_frame_thead_working:
            time.sleep(1/self._fps)

    # ----------------------------------------------------------------------
    def _new_frame(self):
        while self._run:
            _last_time = time.time()
            time.sleep(1 / self._fps)
            if self._generate:

                self._last_frame = self._data[self._picture_size[0]:self._picture_size[2],
                                              self._picture_size[1]:self._picture_size[3]]
                self._new_frame_flag = True
                logger.debug(f"{self._my_name} new frame")

                _last_time = time.time()

        self._new_frame_thead_working = False

    # ----------------------------------------------------------------------
    def _generator(self):
        """
        """
        while self._run:
            _last_time = time.time()
            time.sleep(1 / 10)
            if self._generate:
                nPoints = self.FRAME_W * self.FRAME_H
                self._data = self._baseData + np.random.uniform(0.0, self.NOISE, nPoints).reshape(self.FRAME_W, self.FRAME_H)
                _last_time = time.time()

        self._generator_thread_working = False

    # ----------------------------------------------------------------------
    def start_acquisition(self):

        logger.debug(f"{self._my_name} starting thread")

        self._generate = True
        return True

    # ----------------------------------------------------------------------
    def stop_acquisition(self):
        self._generate = False

    # ----------------------------------------------------------------------
    def get_settings(self, option, cast, do_rotate=True):

        if option in ['FPSmax', 'max_width', 'max_height']:

            logger.debug(f'{self._my_name}: setting {cast.__name__}({option}) requested')

            if option == 'FPSmax':
                return 200
            elif option == 'max_width':
                return self.FRAME_W
            elif option == 'max_height':
                return self.FRAME_H
        else:
            return super(DummyProxy, self).get_settings(option, cast, do_rotate)

    # ----------------------------------------------------------------------
    def save_settings(self, option, value):

        if option == 'FPS':

            logger.debug(f'{self._my_name}: setting {option}: new value {value}')
            self._fps = value

        super(DummyProxy, self).save_settings(option, value)


import sys
import os
import logging
import traceback
import time

try:
    from cStringIO import StringIO
except ImportError:
    from io import StringIO

from pathlib import Path
from PyQt5 import QtWidgets
from optparse import OptionParser

from petra_camera.main_window import PETRACamera, APP_NAME
from .version import __version__


# --------------------------------------------------------------------
def excepthook(exc_type, exc_value, traceback_obj):
    """
    Global function to catch unhandled exceptions. This function will result in an error dialog which displays the
    error information.

    :param exc_type: exception type
    :param exc_value: exception value
    :param traceback_obj: traceback object
    :return:
    """
    separator = '-' * 80
    log_path = f"{os.path.expanduser('~')}/.petra_camera/error.log"
    time_string = time.strftime("%Y-%m-%d, %H:%M:%S")
    tb_info_file = StringIO()
    traceback.print_tb(traceback_obj, None, tb_info_file)
    tb_info_file.seek(0)
    tb_info = tb_info_file.read()
    errmsg = '%s: \n%s' % (str(exc_type), str(exc_value))
    sections = [separator, time_string, separator, errmsg, separator, tb_info]
    msg = '\n'.join(sections)
    try:
        f = open(log_path, "a")
        f.write(msg)
        f.close()
    except IOError:
        pass

    msg_box = QtWidgets.QMessageBox()
    msg_box.setModal(False)
    msg_box.setIcon(QtWidgets.QMessageBox.Critical)
    msg_box.setText(msg)
    msg_box.setInformativeText(msg)
    msg_box.setWindowTitle("Error")
    msg_box.setStandardButtons(QtWidgets.QMessageBox.Ok)
    msg_box.show()


# --------------------------------------------------------------------
def setup_logger(args):
    log_level = "DEBUG"
    format = (
        "%(asctime)s %(filename)s:%(lineno)d "
        "%(levelname)-8s %(message)s")

    if not os.path.exists(f"{str(Path.home())}/.petra_camera"):
        os.mkdir(f"{str(Path.home())}/.petra_camera")

    filename = f"{str(Path.home())}/.petra_camera/camera.log"
    print(f"Logs to file: {filename}")
    logging.basicConfig(level=log_level, format=format, filename=filename)
    logging.info("Log level set to %s", log_level)

    if args.log:
        console = logging.StreamHandler()
        console.setLevel(logging.DEBUG)
        # set a format which is simpler for console use
        formatter = logging.Formatter("%(asctime)s %(module)s %(lineno)-6d %(levelname)-6s %(message)s")
        console.setFormatter(formatter)
        logging.getLogger(APP_NAME).addHandler(console)


# --------------------------------------------------------------------
def main():

    parser = OptionParser()

    parser.add_option("-p", "--profile", dest='profile', default='default', help="profile selection")
    parser.add_option("--log", action='store_true', dest='log', help="print logs to console")

    (options, _) = parser.parse_args()

    setup_logger(options)

    app = QtWidgets.QApplication([])
    sys.excepthook = excepthook

    mainWindow = PETRACamera(options)
    mainWindow.show()

    app.exec_()
    del app


# --------------------------------------------------------------------
# Start Qt event loop unless running in interactive mode.
if __name__ == '__main__':
    main()



# Created by matveyev at 19.08.2021
"""
  Widget for individual camera.
  Has 4 dockable widgets: Frame viewer, Settings, ROI & Marker, PeakSearch.
  Also has his own datasource instance

"""

import logging

try:
    import skimage
    peak_search = True
except:
    peak_search = False

from functools import partial
from PyQt5 import QtCore, QtWidgets, QtGui

from petra_camera.utils.errors import report_error

from petra_camera.main_window import APP_NAME
from petra_camera.widgets.frame_viewer import FrameViewer
from petra_camera.widgets.settings_widget import SettingsWidget
from petra_camera.widgets.marker_roi import MarkersROIsWidget
from petra_camera.widgets.peak_search_widget import PeakSearchWidget

from petra_camera.devices.datasource2d import DataSource2D
from petra_camera.gui.CameraWidget_ui import Ui_CameraWindow

logger = logging.getLogger(APP_NAME)

# ----------------------------------------------------------------------
class CameraWidget(QtWidgets.QMainWindow):

    REFRESH_PERIOD = 5000 # how often we update settings with Tango

    # ----------------------------------------------------------------------
    def __init__(self, parent, settings, my_name):
        """
        """
        super(CameraWidget, self).__init__(parent)

        self.settings = settings
        self.camera_name = my_name
        self._parent = parent

        self.camera_device = DataSource2D(self)
        state, msg = self.camera_device.new_device_proxy(self.camera_name, self._parent.auto_screen_action.isChecked())
        if not state:
            raise RuntimeError(f'{msg}')

        self._ui = Ui_CameraWindow()
        self._ui.setupUi(self)
        self._init_ui()

        self._refresh_view_timer = QtCore.QTimer(self)
        self._refresh_view_timer.timeout.connect(self._refresh_view)
        self._refresh_view_timer.start(self.REFRESH_PERIOD)

    # ----------------------------------------------------------------------
    def block_hist_signals(self, flag):
        """
            during the picture update in Frame viewer we need to disconnect histogram signals in Setting
        :param flag: bool
        :return:
        """
        self._settings_widget.block_hist_signals(flag)

    # ----------------------------------------------------------------------
    def _start_stop_live_mode(self):
        """

        :return: None
        """

        try:
            self._frame_viewer.start_stop_live_mode(self._parent.auto_screen_action.isChecked())
        except Exception as err:
            self._log.exception(err)
            report_error(err)

    # ----------------------------------------------------------------------
    def clean_close(self):
        """

        :return: None
        """
        logger.info(f"Closing {self.camera_name}...")

        self._settings_widget.close()
        self._frame_viewer.close(self._parent.auto_screen_action.isChecked())

        self.camera_device.close_camera()

        self._frame_viewer.save_ui_settings(self.camera_name)
        self._settings_widget.save_ui_settings(self.camera_name)
        self._markerroi_widget.save_ui_settings(self.camera_name)
        if self._peak_search_widget is not None:
            self._peak_search_widget.save_ui_settings(self.camera_name)

        self._save_ui_settings()

        logger.info(f"{self.camera_name} closed.")

    # ----------------------------------------------------------------------
    def _refresh_view(self):
        """

        :return: None
        """

        try:
            state = self.camera_device.is_running()
            if state is None:
                self._action_start_stop.setIcon(QtGui.QIcon(":/ico/play_16px.png"))
                self._action_start_stop.setEnabled(False)
            elif state:
                self._action_start_stop.setIcon(QtGui.QIcon(":/ico/stop.png"))
                self._action_start_stop.setEnabled(True)
            else:
                self._action_start_stop.setIcon(QtGui.QIcon(":/ico/play_16px.png"))
                self._action_start_stop.setEnabled(True)
        except Exception as err:
            logger.exception(f"Exception: camera state: {err}")

        if self._settings_widget.isVisible():
            self._settings_widget.refresh_view()

    # ----------------------------------------------------------------------
    def _display_fps(self, fps):
        """

        :param fps: value, int
        :return: None
        """
        self._lb_fps.setText("{:.2f} FPS".format(fps))

    # ----------------------------------------------------------------------
    def _viewer_cursor_moved(self, x, y):
        """

        :param x: mouse coordinate
        :param y: mouse coordinate
        :return: None
        """
        self._lb_cursor_pos.setText("({:.2f}, {:.2f})".format(x, y))

    # ----------------------------------------------------------------------
    # -------------------- UI initialization -------------------------------
    # ----------------------------------------------------------------------
    def _init_ui(self):

        self._init_tool_bar()

        self._init_status_bar()

        self._load_docks()

        self._connect_signals()

    # ----------------------------------------------------------------------
    def _load_docks(self):
        """
         here we loads all widgets, dock them
        """
        self.setCentralWidget(None)

        self.setDockOptions(QtWidgets.QMainWindow.AnimatedDocks |
                            QtWidgets.QMainWindow.AllowNestedDocks |
                            QtWidgets.QMainWindow.AllowTabbedDocks)

        self._frame_viewer, self._frameViewer_dock = \
            self._add_dock(FrameViewer, "Frame", self)

        self._settings_widget, self._settings_dock = \
            self._add_dock(SettingsWidget, "Settings", self)

        self._markerroi_widget, self._markerroi_dock = \
            self._add_dock(MarkersROIsWidget, "Markers/ROIs", self)

        if peak_search:
            self._peak_search_widget, self._peak_search_dock = \
                self._add_dock(PeakSearchWidget, "Peak Search", self)
        else:
            self._peak_search_widget, self._peak_search_dock = None, None

            # after all widgets are loaded we restore the user layout
        self._frame_viewer.load_ui_settings(self.camera_name)
        self._settings_widget.load_ui_settings(self.camera_name)
        self._markerroi_widget.load_ui_settings(self.camera_name)
        if self._peak_search_widget is not None:
            self._peak_search_widget.load_ui_settings(self.camera_name)

        # link between picture and histogram
        self._settings_widget.set_frame_to_hist(self._frame_viewer.get_image_view())
        self._frame_viewer.set_hist(self._settings_widget.get_hist())

    # ----------------------------------------------------------------------
    def _connect_signals(self):
        """
            connect signals between widgets
        :return: None
        """

        self._frame_viewer.new_fps.connect(self._display_fps)
        self._frame_viewer.cursor_moved.connect(self._viewer_cursor_moved)

        self.camera_device.new_frame.connect(self._frame_viewer.new_frame)

        self.camera_device.update_roi_statistics.connect(self._markerroi_widget.update_roi_statistics)
        self.camera_device.update_roi_statistics.connect(self._frame_viewer.repaint_roi)

        self.camera_device.update_peak_search.connect(self._frame_viewer.repaint_peak_search)

        self.camera_device.got_error.connect(lambda err_msg: report_error(err_msg, self, True))

        self._markerroi_widget.add_remove_marker.connect(self._frame_viewer.add_remove_marker)
        self._markerroi_widget.repaint_marker.connect(self._frame_viewer.repaint_marker)

        self._markerroi_widget.add_remove_roi.connect(self._frame_viewer.add_remove_roi)
        self._markerroi_widget.repaint_roi.connect(self._frame_viewer.repaint_roi)

        self._action_start_stop.triggered.connect(self._start_stop_live_mode)
        self._action_print_image.triggered.connect(self._frame_viewer.print_image)
        self._action_copy_image.triggered.connect(self._frame_viewer.to_clipboard)

        self._saveImgAction.triggered.connect(self._frame_viewer.save_to_image)
        self._saveAsciiAction.triggered.connect(partial(self._frame_viewer.save_to_file, fmt="csv"))
        self._saveNumpyAction.triggered.connect(partial(self._frame_viewer.save_to_file, fmt="npy"))

    # ----------------------------------------------------------------------
    def _add_dock(self, WidgetClass, label, *args, **kwargs):

        widget = WidgetClass(*args, **kwargs)

        dock = QtWidgets.QDockWidget(label)
        dock.setObjectName("{0}Dock".format("".join(label.split())))
        dock.setStyleSheet("""QDockWidget {font-size: 11pt; font-weight: normal}""")
        dock.setWidget(widget)

        self.addDockWidget(QtCore.Qt.LeftDockWidgetArea, dock)

        self.tool_bar.addAction(dock.toggleViewAction())

        return widget, dock

    # ----------------------------------------------------------------------
    def _make_save_menu(self, parent):
        """
        Args:
            parent (QWidget)
        """
        saveMenu = QtWidgets.QMenu(parent)

        self._saveImgAction = saveMenu.addAction("Image")

        self._saveAsciiAction = saveMenu.addAction("ASCII")

        self._saveNumpyAction = saveMenu.addAction("Numpy")

        return saveMenu

    # ----------------------------------------------------------------------
    def _init_tool_bar(self):
        """
        """

        self.tool_bar = QtWidgets.QToolBar("Main toolbar", self)
        self.tool_bar.setObjectName("CameraViewer_ToolBar")

        self._action_start_stop = QtWidgets.QAction(self)
        self._action_start_stop.setIcon(QtGui.QIcon(":/ico/play_16px.png"))
        self._action_start_stop.setText("Start/Stop")

        self.tool_bar.addAction(self._action_start_stop)
        self.tool_bar.addSeparator()

        # image saving
        self._tbSaveScan = QtWidgets.QToolButton(self)
        self._tbSaveScan.setIcon(QtGui.QIcon(":/ico/save.png"))
        self._tbSaveScan.setToolTip("Save")

        self._saveMenu = self._make_save_menu(self._tbSaveScan)
        self._tbSaveScan.setMenu(self._saveMenu)
        self._tbSaveScan.setPopupMode(QtWidgets.QToolButton.InstantPopup)
        self.tool_bar.addWidget(self._tbSaveScan)

        self._action_print_image = QtWidgets.QAction(self)
        self._action_print_image.setIcon(QtGui.QIcon(":/ico/print.png"))
        self._action_print_image.setText("Print Image")
        self._action_print_image.setEnabled(False)
        self.tool_bar.addAction(self._action_print_image)

        self._action_copy_image = QtWidgets.QAction(self)
        self._action_copy_image.setIcon(QtGui.QIcon(":/ico/copy.png"))
        self._action_copy_image.setText("Copy to Clipboard")
        self.tool_bar.addAction(self._action_copy_image)

        self.tool_bar.addSeparator()
        self.tool_bar.addWidget(QtWidgets.QLabel('Widgets:'))

        self.addToolBar(self.tool_bar)

    # ----------------------------------------------------------------------
    def _init_status_bar(self):

        self._lb_cursor_pos = QtWidgets.QLabel("")

        self._lb_fps = QtWidgets.QLabel("FPS: -")
        self._lb_fps.setMinimumWidth(70)

        self.statusBar().addPermanentWidget(self._lb_cursor_pos)
        self.statusBar().addPermanentWidget(self._lb_fps)

    # ----------------------------------------------------------------------
    def _save_ui_settings(self):
        """Save basic GUI settings.
        """
        settings = QtCore.QSettings(APP_NAME)

        settings.setValue(f"{self.camera_name}/geometry", self.saveGeometry())
        settings.setValue(f"{self.camera_name}/state", self.saveState())

    # ----------------------------------------------------------------------
    def load_ui_settings(self):
        """Load basic GUI settings.
        """
        settings = QtCore.QSettings(APP_NAME)

        try:
            self.restoreGeometry(settings.value(f"{self.camera_name}/geometry"))
        except:
            pass

        try:
            self.restoreState(settings.value(f"{self.camera_name}/state"))
        except:
            pass



# Created by matveyev at 19.08.2021

import logging

from PyQt5 import QtCore, QtWidgets

from petra_camera.main_window import APP_NAME

WIDGET_NAME = None
SAVE_STATE_UIS = []

logger = logging.getLogger(APP_NAME)


# ----------------------------------------------------------------------
class BaseWidget(QtWidgets.QWidget):

    # ----------------------------------------------------------------------
    def __init__(self, camera_widget):

        super(BaseWidget, self).__init__(camera_widget)

        self._parent = camera_widget
        self._settings = camera_widget.settings
        self._camera_device = camera_widget.camera_device

    # ----------------------------------------------------------------------
    def save_ui_settings(self, camera_name):
        """
        """
        settings = QtCore.QSettings(APP_NAME)
        for uis in SAVE_STATE_UIS:
            settings.setValue(f"{WIDGET_NAME}_{camera_name}/f{uis}", getattr(self._ui, uis).saveState())

        settings.setValue(f"{WIDGET_NAME}_{camera_name}/geometry", self.saveGeometry())

    # ----------------------------------------------------------------------
    def load_ui_settings(self, camera_name):
        """
        """
        settings = QtCore.QSettings(APP_NAME)
        for uis in SAVE_STATE_UIS:
            try:
                getattr(self._ui, uis).restoreState(settings.value(f"{WIDGET_NAME}_{camera_name}/f{uis}"))
            except:
                pass

        try:
            self.restoreGeometry(settings.value(f"{WIDGET_NAME}_{camera_name}/geometry"))
        except:
            pass



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'uis/Marker.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Marker(object):
    def setupUi(self, Marker):
        Marker.setObjectName("Marker")
        Marker.resize(309, 41)
        self.horizontalLayout = QtWidgets.QHBoxLayout(Marker)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.chk_visible = QtWidgets.QCheckBox(Marker)
        self.chk_visible.setText("")
        self.chk_visible.setObjectName("chk_visible")
        self.horizontalLayout.addWidget(self.chk_visible)
        self.label_23 = QtWidgets.QLabel(Marker)
        self.label_23.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_23.setObjectName("label_23")
        self.horizontalLayout.addWidget(self.label_23)
        self.sb_x = QtWidgets.QSpinBox(Marker)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sb_x.sizePolicy().hasHeightForWidth())
        self.sb_x.setSizePolicy(sizePolicy)
        self.sb_x.setMaximum(99999)
        self.sb_x.setObjectName("sb_x")
        self.horizontalLayout.addWidget(self.sb_x)
        self.label_24 = QtWidgets.QLabel(Marker)
        self.label_24.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_24.setObjectName("label_24")
        self.horizontalLayout.addWidget(self.label_24)
        self.sb_y = QtWidgets.QSpinBox(Marker)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sb_y.sizePolicy().hasHeightForWidth())
        self.sb_y.setSizePolicy(sizePolicy)
        self.sb_y.setMaximum(99999)
        self.sb_y.setObjectName("sb_y")
        self.horizontalLayout.addWidget(self.sb_y)
        self.but_color = QtWidgets.QPushButton(Marker)
        self.but_color.setMinimumSize(QtCore.QSize(23, 23))
        self.but_color.setMaximumSize(QtCore.QSize(23, 23))
        self.but_color.setText("")
        self.but_color.setObjectName("but_color")
        self.horizontalLayout.addWidget(self.but_color)
        self.but_delete = QtWidgets.QPushButton(Marker)
        self.but_delete.setObjectName("but_delete")
        self.horizontalLayout.addWidget(self.but_delete)

        self.retranslateUi(Marker)
        QtCore.QMetaObject.connectSlotsByName(Marker)

    def retranslateUi(self, Marker):
        _translate = QtCore.QCoreApplication.translate
        Marker.setWindowTitle(_translate("Marker", "Form"))
        self.label_23.setText(_translate("Marker", "X:"))
        self.label_24.setText(_translate("Marker", "Y:"))
        self.but_color.setToolTip(_translate("Marker", "color"))
        self.but_delete.setText(_translate("Marker", "Delete"))




# Created by matveyev at 22.02.2022

import logging

from PyQt5 import QtWidgets, QtGui, QtCore

from petra_camera.main_window import APP_NAME
from petra_camera.widgets.camera_settings import CameraSettings
from petra_camera.gui.SettingsDialog_ui import Ui_SettingsDialog

WIDGET_NAME = 'ProgramSetup'

logger = logging.getLogger(APP_NAME)


class ProgramSetup(QtWidgets.QDialog):

    # ----------------------------------------------------------------------
    def __init__(self, main_window):
        """
        """
        super(ProgramSetup, self).__init__()
        self._ui = Ui_SettingsDialog()
        self._ui.setupUi(self)

        self._main_window = main_window

        self._last_camera_id = 0

        self._settings = main_window.settings

        self.btn_add_camera = QtWidgets.QToolButton(self)
        self.btn_add_camera.setText('Add camera') #.setIcon(QtGui.QIcon(":/icons/plus_small.png"))
        self.btn_add_camera.clicked.connect(self._add_camera)
        self._ui.tb_cameras.setCornerWidget(self.btn_add_camera, QtCore.Qt.TopRightCorner)
        self._ui.tb_cameras.cornerWidget().setMinimumSize(self.btn_add_camera.sizeHint())

        # self._ui.cmd_sav_profile.clicked.connect(self._save_settings)
        # self._ui.cmd_load_profile.clicked.connect(self._load_settings)

        self._ui.cmd_save_folder.clicked.connect(self._new_save_folder)

        self._ui.cmd_roi_frame_color.clicked.connect(lambda: self._pick_color('cmd_roi_frame_color'))
        self._ui.cmd_roi_bkg_color.clicked.connect(lambda: self._pick_color('cmd_roi_bkg_color'))
        self._ui.cmd_marker_color.clicked.connect(lambda: self._pick_color('cmd_marker_color'))
        self._ui.cmd_title_bkg_color.clicked.connect(lambda: self._pick_color('cmd_title_bkg_color'))
        self._ui.cmd_roi_label_font_color.clicked.connect(lambda: self._pick_color('cmd_roi_label_font_color'))
        self._ui.cmd_title_font_color.clicked.connect(lambda: self._pick_color('cmd_title_font_color'))

        self._ui.cmd_roi_label_font.clicked.connect(lambda: self._pick_font('cmd_roi_label_font'))
        self._ui.cmd_title_font.clicked.connect(lambda: self._pick_font('cmd_title_font'))

        self._display_settings()

    # ----------------------------------------------------------------------
    def _new_save_folder(self):

        folder = QtWidgets.QFileDialog.getExistingDirectory(self, 'Select safe folder', self._ui.le_save_path.text())
        if folder:
            self._ui.le_save_path.setText(folder)

    # ----------------------------------------------------------------------
    def _display_settings(self):
        self._ui.le_save_path.setText(self._settings.option("save_folder", "default"))

        self._ui.cmd_roi_frame_color.setStyleSheet("QPushButton {background-color: " +
                                                   f"{self._settings.option('roi', 'fr_color')}" + ";}")

        self._ui.cmd_roi_label_font.setText(self._settings.option('roi', 'font'))
        font = self._settings.option('roi', 'font').split(',')
        self._ui.cmd_roi_label_font.setFont(QtGui.QFont(font[0], int(font[1])))

        self._ui.cmd_roi_label_font_color.setStyleSheet("QPushButton {background-color: " +
                                                   f"{self._settings.option('roi', 'fg_color')}" + ";}")

        self._ui.cmd_roi_bkg_color.setStyleSheet("QPushButton {background-color: " +
                                                 f"{self._settings.option('roi', 'bg_color')}" + ";}")

        self._ui.cmd_marker_color.setStyleSheet("QPushButton {background-color: " +
                                                f"{self._settings.option('marker', 'fr_color')}" + ";}")

        self._ui.cmd_title_font.setText(self._settings.option('title', 'font'))
        font = self._settings.option('title', 'font').split(',')
        self._ui.cmd_title_font.setFont(QtGui.QFont(font[0], int(font[1])))

        self._ui.cmd_title_font_color.setStyleSheet("QPushButton {background-color: " +
                                                    f"{self._settings.option('title', 'fg_color')}" + ";}")
        self._ui.cmd_title_bkg_color.setStyleSheet("QPushButton {background-color: " +
                                                   f"{self._settings.option('title', 'bg_color')}" + ";}")

        if self._settings.has_node('roi_server') and self._settings.option("roi_server", "enable").lower() == "true":
            self._ui.chk_roi_server_enable.setChecked(True)
            self._ui.le_roi_server_host.setEnabled(True)
            self._ui.le_roi_server_host.setText(self._settings.option("roi_server", "host"))
            self._ui.le_roi_server_port.setEnabled(True)
            self._ui.le_roi_server_port.setText(self._settings.option("roi_server", "port"))
        else:
            self._ui.chk_roi_server_enable.setChecked(False)
            self._ui.le_roi_server_host.setEnabled(False)
            self._ui.le_roi_server_host.setEnabled(False)

        self._ui.dsb_cross_size.setValue(float(self._settings.option("center_search", "cross")))
        self._ui.dsb_circle_size.setValue(float(self._settings.option("center_search", "circle")))

        for ind, device in enumerate(self._settings.get_nodes('camera')):
            widget = CameraSettings(self, ind, device)
            widget.delete_me.connect(self._delete_camera)
            widget.new_name.connect(self._new_name)
            self._ui.tb_cameras.addTab(widget, device.get('name'))
            self._last_camera_id += 1

    # ----------------------------------------------------------------------
    def _add_camera(self):
        widget = CameraSettings(self, self._last_camera_id)
        widget.delete_me.connect(self._delete_camera)
        widget.new_name.connect(self._new_name)
        self._ui.tb_cameras.addTab(widget, 'New camera')
        self._last_camera_id += 1

    # ----------------------------------------------------------------------
    def _delete_camera(self, id_to_delete):
        tab_to_delete = None
        for ind in range(self._ui.tb_cameras.count()):
            if self._ui.tb_cameras.widget(ind).my_id == id_to_delete:
                tab_to_delete = ind

        if tab_to_delete is not None:
            self._ui.tb_cameras.removeTab(tab_to_delete)

    # ----------------------------------------------------------------------
    def _new_name(self, id, new_name):
        tab_to_rename = None
        for ind in range(self._ui.tb_cameras.count()):
            if self._ui.tb_cameras.widget(ind).my_id == id:
                tab_to_rename = ind

        if tab_to_rename is not None:
            self._ui.tb_cameras.setTabText(tab_to_rename, new_name)

    # ----------------------------------------------------------------------
    def _pick_color(self, source):

        color = QtWidgets.QColorDialog.getColor(getattr(self._ui, source).palette().button().color())

        if color.isValid():
            getattr(self._ui, source).setStyleSheet("QPushButton {background-color: " + f"{color.name()}" + ";}")

    # ----------------------------------------------------------------------
    def _pick_font(self, source):

        font = QtWidgets.QFontDialog.getFont(getattr(self._ui, source).font())

        if font[1]:
            getattr(self._ui, source).setFont(font[0])
            getattr(self._ui, source).setText(font[0].family() + ', ' + str(font[0].pointSize()))

    # ----------------------------------------------------------------------
    def _apply_settings(self):
        general_options = []

        general_options.append(("save_folder", (("default", self._ui.le_save_path.text()),)))

        font = self._ui.cmd_roi_label_font.font()
        general_options.append(('roi', (('font', font.family() + ',' + str(font.pointSize())),
                                     ('fg_color', self._ui.cmd_roi_label_font_color.palette().button().color().name()),
                                     ('bg_color', self._ui.cmd_roi_bkg_color.palette().button().color().name()),
                                     ('fr_color', self._ui.cmd_roi_frame_color.palette().button().color().name()))))

        general_options.append(('marker', (('fr_color', self._ui.cmd_marker_color.palette().button().color().name()),)))

        font = self._ui.cmd_title_font.font()
        general_options.append(('title', (('font', font.family() + ',' + str(font.pointSize())),
                                       ('fg_color', self._ui.cmd_title_font_color.palette().button().color().name()),
                                       ('bg_color', self._ui.cmd_title_bkg_color.palette().button().color().name()))))

        general_options.append(("roi_server", (("enable", 'true' if self._ui.chk_roi_server_enable.isChecked() else 'false'),
                                            ("host", self._ui.le_roi_server_host.text()),
                                            ("port", self._ui.le_roi_server_port.text()))))

        general_options.append(("center_search", (("cross", self._ui.dsb_cross_size.value()),
                                               ("circle", self._ui.dsb_circle_size.value()))))


        cameras_settings = []
        for ind in range(self._ui.tb_cameras.count()):
            cameras_settings.append(self._ui.tb_cameras.widget(ind).get_data())

        self._settings.set_options(general_options, cameras_settings)

    # ----------------------------------------------------------------------
    def accept(self):

        self._apply_settings()

        QtCore.QSettings(APP_NAME).setValue("{}/geometry".format(WIDGET_NAME), self.saveGeometry())

        super(ProgramSetup, self).accept()

    # ----------------------------------------------------------------------
    def reject(self):

        QtCore.QSettings(APP_NAME).setValue("{}/geometry".format(WIDGET_NAME), self.saveGeometry())

        super(ProgramSetup, self).reject()


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'uis/CameraWidget.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_CameraWindow(object):
    def setupUi(self, CameraWindow):
        CameraWindow.setObjectName("CameraWindow")
        CameraWindow.resize(845, 480)
        self.centralwidget = QtWidgets.QWidget(CameraWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        CameraWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(CameraWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 845, 21))
        self.menubar.setObjectName("menubar")
        CameraWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(CameraWindow)
        self.statusbar.setSizeGripEnabled(False)
        self.statusbar.setObjectName("statusbar")
        CameraWindow.setStatusBar(self.statusbar)
        self.quit_action = QtWidgets.QAction(CameraWindow)
        self.quit_action.setObjectName("quit_action")
        self.actionAbout = QtWidgets.QAction(CameraWindow)
        self.actionAbout.setObjectName("actionAbout")

        self.retranslateUi(CameraWindow)
        QtCore.QMetaObject.connectSlotsByName(CameraWindow)

    def retranslateUi(self, CameraWindow):
        _translate = QtCore.QCoreApplication.translate
        CameraWindow.setWindowTitle(_translate("CameraWindow", "CameraWindow"))
        self.quit_action.setText(_translate("CameraWindow", "Quit..."))
        self.quit_action.setShortcut(_translate("CameraWindow", "Ctrl+Q"))
        self.actionAbout.setText(_translate("CameraWindow", "About..."))

import petra_camera.gui.icons_rc



# ----------------------------------------------------------------------
# Author:        yury.matveev@desy.de
# ----------------------------------------------------------------------

"""Vimba camera proxy
"""

import time
import numpy as np
import logging

try:
    import PyTango
except ImportError:
    pass

from threading import Thread
from distutils.util import strtobool

from petra_camera.devices.base_camera import BaseCamera
from petra_camera.main_window import APP_NAME

logger = logging.getLogger(APP_NAME)


# ----------------------------------------------------------------------
class VimbaProxy(BaseCamera):
    """Proxy to a physical TANGO device.
    """
    SERVER_SETTINGS = {'bw':    {'low': [1, 'Image8'],
                                 'high': [2, 'Image16']},
                       'color': {'low': [5, 'ImageRGB'],
                                 'high': [5, 'ImageRGB']}
                       }

    PIXEL_FORMATS = {'color': {'high': [],
                               'low': ['RGB8Packed']},
                     'bw':    {'high': ["Mono12", "BayerGR12", "BayerRG12", "BayerGB12"],
                               'low': ['Mono8']}
                     }

    START_DELAY = 1
    STOP_DELAY = 0.5

    START_ATTEMPTS = 2

    _settings_map = {"exposure": ("device_proxy", "ExposureTimeAbs"),
                     "gain": ["device_proxy", "Gain"],
                     'FPSmax': ("device_proxy", "AcquisitionFrameRateLimit"),
                     'FPS': ("device_proxy", "AcquisitionFrameRateAbs"),
                     'view_x': ("device_proxy", "OffsetX"),
                     'view_y': ("device_proxy", "OffsetY"),
                     'view_h': ("device_proxy", "Height"),
                     'view_w': ("device_proxy", "Width"),
                     'max_width': ("device_proxy", "WidthMax"),
                     'max_height': ("device_proxy", "HeightMax")
                     }

    visible_layouts = ('FPS', 'exposure')

    # ----------------------------------------------------------------------
    def __init__(self, settings):
        super(VimbaProxy, self).__init__(settings)

        self._settings_map["gain"][1] = str(self._device_proxy.get_property('GainFeatureName')['GainFeatureName'][0])
        if 'high_depth' in settings.keys():
            high_depth = strtobool(settings.get("high_depth"))
        else:
            high_depth = False

        if 'color' in settings.keys():
            color = strtobool(settings.get("color"))
        else:
            color = False

        accepted_format = None

        if self._device_proxy.state() != PyTango.DevState.MOVING:
            valid_formats = self._device_proxy.read_attribute('PixelFormat_Values').value
        else:
            valid_formats = self._device_proxy.pixelformat
        if color:
            if high_depth:
                for pixel_format in self.PIXEL_FORMATS['color']['high']:
                    if pixel_format in valid_formats:
                        accepted_format = pixel_format
                        self._high_depth = True
                        self._color = True
            if accepted_format is None:
                for pixel_format in self.PIXEL_FORMATS['color']['low']:
                    if pixel_format in valid_formats:
                        accepted_format = pixel_format
                        self._high_depth = False
                        self._color = True

        if accepted_format is None:
            if high_depth:
                for pixel_format in self.PIXEL_FORMATS['bw']['high']:
                    if pixel_format in valid_formats:
                        accepted_format = pixel_format
                        self._high_depth = True
                        self._color = False
            if accepted_format is None:
                for pixel_format in self.PIXEL_FORMATS['bw']['low']:
                    if pixel_format in valid_formats:
                        accepted_format = pixel_format
                        self._high_depth = False
                        self._color = False

        if accepted_format is None:
            raise RuntimeError('Cannot find acceptable pixel format!')

        if self._device_proxy.state() != PyTango.DevState.MOVING:
            self._device_proxy.pixelformat = accepted_format
            self._device_proxy.viewingmode = \
            self.SERVER_SETTINGS['color' if self._color else 'bw']['high' if self._high_depth else 'low'][0]

        self._image_source = \
            self.SERVER_SETTINGS['color' if self._color else 'bw']['high' if self._high_depth else 'low'][1]

        self.error_msg = ''
        self.error_flag = False
        self._last_frame = np.zeros((1, 1))
        self._last_time = time.time()

        self._mode = 'event'
        self._eid = None
        self._frame_thread = None
        self._frame_thread_running = False
        self._stop_frame_thread = False

    # ----------------------------------------------------------------------
    def get_settings(self, option, cast, do_rotate=True):

        if option == 'max_level_limit':

            logger.debug(f'{self._my_name}: setting {cast.__name__}({option}) requested')

            if self._high_depth:
                return 2 ** 12
            else:
                return 2 ** 8

        elif option == 'exposure':
            return super(VimbaProxy, self).get_settings(option, cast, do_rotate) / 1000

        else:
            return super(VimbaProxy, self).get_settings(option, cast, do_rotate)

    # ----------------------------------------------------------------------
    def save_settings(self, option, value):

        if option == 'exposure':
            super(VimbaProxy, self).save_settings(option, value*1000)
        else:
            super(VimbaProxy, self).save_settings(option, value)

    # ----------------------------------------------------------------------
    def start_acquisition(self):
        """
        """

        if self._device_proxy.state() == PyTango.DevState.ON:

            logger.debug(f'{self._my_name}: starting acquisition: event mode')

            self._mode = 'event'
            self._eid = self._device_proxy.subscribe_event(self._image_source,
                                                           PyTango.EventType.DATA_READY_EVENT,
                                                           self._readout_frame, [], True)

            attemp = 0
            while attemp < self.START_ATTEMPTS:
                try:
                    self._device_proxy.command_inout("StartAcquisition")
                    break
                except:
                    attemp += 1

            time.sleep(self.START_DELAY)  # ? TODO

            return True
        elif self._device_proxy.state() == PyTango.DevState.MOVING:

            logger.debug(f'{self._my_name}: starting acquisition: thread mode')

            self._stop_frame_thread = False
            self._mode = 'attribute'
            self._frame_thread = Thread(target=self._read_frame)
            self._frame_thread_running = True
            self._frame_thread.start()
            return True

        else:
            logger.warning("Camera should be in ON state (is it running already?)")
            return False

    # ----------------------------------------------------------------------
    def stop_acquisition(self):
        """
        """
        if self._mode == 'event':
            if self._eid is not None:
                self._device_proxy.unsubscribe_event(self._eid)
                self._eid = None
            if self._device_proxy.state() == PyTango.DevState.MOVING:
                self._device_proxy.command_inout("StopAcquisition")
        else:
            if self._frame_thread_running:
                self._stop_frame_thread = True
                while self._frame_thread_running:
                    time.sleep(0.1)

            time.sleep(self.STOP_DELAY)  # ? TODO

    # ----------------------------------------------------------------------
    def is_running(self):
        if self._mode == 'event':
            return self._device_proxy.state() == PyTango.DevState.MOVING
        else:
            if self._device_proxy.state() != PyTango.DevState.MOVING:
                return False
            else:
                return self._frame_thread_running

    # ----------------------------------------------------------------------
    def _read_frame(self):
        sleep_time = self.get_settings('FPS', int)
        while not self._stop_frame_thread:
            if self._device_proxy.state() != PyTango.DevState.MOVING:
                self._frame_thread_running = False
                raise RuntimeError('Camera was stopped!')
            try:
                self._last_frame = self._process_frame(getattr(self._device_proxy, self._image_source))
                self._new_frame_flag = True
            except Exception as err:
                logger.error('Vimba error: {}'.format(err))
                self.error_flag = True
                self.error_msg = str(err)
            time.sleep(1/sleep_time)
        self._frame_thread_running = False

    # ----------------------------------------------------------------------
    def _readout_frame(self, event):
        """Called each time new frame is available.
        """
        if not event.err:
            try:
                data = event.device.read_attribute(event.attr_name.split('/')[6])
                self._last_frame = self._process_frame(data.value)
                self._new_frame_flag = True

            except Exception as err:
                logger.error('Vimba error: {}'.format(err))
                self.error_flag = True
                self.error_msg = str(err)
        else:
            logger.error('Vimba error: {}'.format(self.error_msg))
            self.error_flag = True
            self.error_msg = event.errors

    # ----------------------------------------------------------------------
    def _process_frame(self, data):
        data = np.transpose(data)
        if self._color:
            c_data = np.zeros(data.shape + (3,), dtype=np.ubyte)
            c_data[..., 0] = data & 255
            c_data[..., 1] = (data >> 8) & 255
            c_data[..., 2] = (data >> 16) & 255

            return c_data
        else:
            return data

    # ----------------------------------------------------------------------
    def close_camera(self):
        super(VimbaProxy, self).close_camera()

        if self._eid is not None:
            self._device_proxy.unsubscribe_event(self._eid)

        if self._frame_thread_running:
            self._stop_frame_thread = True
            while self._frame_thread_running:
                time.sleep(0.1)




# Created by matveyev at 13.11.2020
from pyqtgraph import ImageView

# ----------------------------------------------------------------------
class ImageViewNoKeyboard(ImageView):
    """
    """
    def keyPressEvent(self, ev):
        pass


# ----------------------------------------------------------------------
# Author:        yury.matveev@desy.de
# ----------------------------------------------------------------------

"""Displays x/y projection of a data frame.
"""

import numpy as np
import pyqtgraph as pg

from PyQt5 import QtCore, QtWidgets, QtGui

from petra_camera.gui.ProjectionWidget_ui import Ui_ProjectionWidget

# ----------------------------------------------------------------------
class ProjectionWidget(QtWidgets.QWidget):
    """
    """
    PLOT_COLOR = QtGui.QColor(80, 90, 210)
   
    cursor_moved = QtCore.pyqtSignal(float, float)
    
    # ----------------------------------------------------------------------
    def __init__(self, parent):
        """
        """
        super(ProjectionWidget, self).__init__(parent)         # ???? TODO

            # temp? TODO
        self.parent = parent    
       
        pg.setConfigOption("leftButtonPan", False)
        
        self._ui = Ui_ProjectionWidget()
        self._ui.setupUi(self)
      
        self._plotItem, self._plot = self._setup_plot()
        self._ui.graphicsView.setCentralItem(self._plotItem)

        self._plotItem.scene().sigMouseMoved.connect(self._mouse_moved)
        self._plotItem.scene().sigMouseClicked.connect(self._mouse_clicked)
 
    # ----------------------------------------------------------------------
    def range_changed(self, array, axis, roiRect):
        """
        """
        x, y, w, h = roiRect

        yVec = array.sum(axis=axis)
        if axis == 0:           # y-profile
            xVec = np.linspace(y, y + h, len(yVec)) #* (-1)
        else:
            xVec = np.linspace(x, x + w, len(yVec))

        self._plot.setData(xVec, yVec)

    # ----------------------------------------------------------------------
    def as_projection_y(self):
        """
        """
        self._plot.rotate(90)

        self._plot.getViewBox().invertY(True)

    # ----------------------------------------------------------------------
    def _mouse_moved(self, pos):
        """
        """
        if self._plotItem.sceneBoundingRect().contains(pos):
            pos = self._plotItem.vb.mapSceneToView(pos)
            self.cursor_moved.emit(pos.x(), pos.y())

    # ----------------------------------------------------------------------
    def _mouse_clicked(self, event):
        """
        """
        if event.double():
            self._plotItem.autoRange()

    # ----------------------------------------------------------------------
    def _setup_plot(self):
        """
        """
        item = pg.PlotItem()
        item.showGrid(True, True)
        item.setMenuEnabled(False)
        item.enableAutoRange()
    
        plot = item.plot([], pen=self.PLOT_COLOR, name="")
 
        return item, plot

